//
// PrintHandler.h
//
// $Id$
//
// Library: JSON
// Package: JSON
// Module:  PrintHandler
//
// Definition of the PrintHandler class.
//
// Copyright (c) 2012, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
//
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#ifndef JSON_PrintHandler_INCLUDED
#define JSON_PrintHandler_INCLUDED


#include "Poco/JSON/JSON.h"
#include "Poco/JSON/Handler.h"


namespace Poco {
namespace JSON {


class JSON_API PrintHandler : public Handler
	/// PrintHandler formats and prints the JSON object
	/// to either user-provided std::ostream or standard out.
	/// If indent i zero, the output is condensed JSON string,
	/// otherwise, the proper indentation is applied to elements.
{
public:
	typedef SharedPtr<PrintHandler> Ptr;

	static const unsigned JSON_PRINT_FLAT = 0;

	PrintHandler(unsigned indent = 0);
		/// Creates the PrintHandler.

	PrintHandler(std::ostream& out, unsigned indent = 0);
		/// Creates the PrintHandler.

	~PrintHandler();
		/// Destroys the PrintHandler.

	void reset();
		/// Resets the handler state.

	void startObject();
		/// The parser has read a '{'; a new object is started.
		/// If indent is greater than zero, a newline will be apended.

	void endObject();
		/// The parser has read a '}'; the object is closed.

	void startArray();
		/// The parser has read a [; a new array will be started.
		/// If indent is greater than zero, a newline will be apended.

	void endArray();
		/// The parser has read a ]; the array is closed.

	void key(const std::string& k);
		/// A key of an object is read; it will be written to the output,
		/// followed by a ':'. If indent is greater than zero, the colon
		/// is padded by a space before and after.

	void null();
		/// A null value is read; "null" will be written to the output.

	void value(int v);
		/// An integer value is read.

	void value(unsigned v);
		/// An unsigned value is read. This will only be triggered if the
		/// value cannot fit into a signed int.
		
#if defined(POCO_HAVE_INT64)
	void value(Int64 v);
		/// A 64-bit integer value is read; it will be written to the output.

	void value(UInt64 v);
		/// An unsigned 64-bit integer value is read; it will be written to the output.
#endif

	void value(const std::string& value);
		/// A string value is read; it will be fromatted and written to the output.

	void value(double d);
		/// A double value is read; it will be written to the output.

	void value(bool b);
		/// A boolean value is read; it will be written to the output.

	void comma();
		/// A comma is read; it will be written to the output as "true" or "false".

	void setIndent(unsigned indent);
		/// Sets indentation.

private:

	const char* endLine() const;
	unsigned indent();
	bool printFlat() const;
	void arrayValue();

	std::ostream& _out;
	unsigned      _indent;
	std::string   _tab;
	bool          _array;
	bool          _value;
};


inline void PrintHandler::setIndent(unsigned indent)
{
	_indent = indent;
}


}} // namespace Poco::JSON


#endif // JSON_PrintHandler_INCLUDED
